@@ -312,6 +312,14 @@ job is either a job number or a job id. Uses current job if not specified."""
 Downloads and stores all the inputs and outputs of a job, as well as the details/status of the job.
 job is either a job number or a job id. Uses current job if not specified."""
 
+        def store_details(job_details):
+            # We don't want to store all the outputs in the details themselves, so
+            # we make a copy and filter that.
+            filtered_details = job_details.copy()
+            self.lib.filter_large_chunks_from_details(filtered_details)
+            filtered_details = pprint.pformat(filtered_details)
+            self.save_content(job_id, 'details.json', filtered_details, text=True)
+
         def store_inline_inputs(job_details):
             inputs = self.lib.get_inputs(job_details, tabular_as_csv=True)
             for name in inputs:
@@ -335,6 +343,10 @@ job is either a job number or a job id. Uses current job if not specified."""
 
         job_id = self._get_and_remember_job_id(job_id)
         details = self.lib.get_job_details(job_id, with_contents='full')
+
+        # We want to store the details first, in case an error happens later
+        # with the assets.
+        store_details(details)
         store_inline_inputs(details)
         store_inline_outputs(details)
         download_data_assets(self.lib.get_output_asset_ids(details))
@@ -342,10 +354,6 @@ job is either a job number or a job id. Uses current job if not specified."""
         # the user to cancel the transfer of these items, which probably would
         # be the largest for a job, after the others have been stored.
         download_data_assets(self.lib.get_input_asset_ids(details))
-        # We don't want to store all the outputs in the details themselves
-        self.lib.filter_large_chunks_from_details(details)
-        details = pprint.pformat(details)
-        self.save_content(job_id, 'details.json', details, text=True)
 
     def do_details(self, arguments, printer=pprint.pprint):
         """details [job] [names|full]
@@ -363,29 +371,31 @@ job is either a job number or a job id. Uses current job if not specified."""
         details = self.lib.get_job_details(job_id, with_contents=with_contents)
         printer(details, indent=4, width=120)
 
+    def parse_job_spec(self, job_spec):
+        """Returns a list of all the job ids that fit the given specification."""
+        if job_spec == '*':
+            # Update the list of jobs before trying to delete them
+            self._cache_jobs()
+            return self.jobs.copy()
+        elif m := re.fullmatch('(\\d+)-(\\d+)', job_spec):
+            n1 = int(m.group(1))
+            n2 = int(m.group(2))
+            # We must gather the list of all the ids of jobs we want to delete
+            # right from the start, because deleting each job will update the
+            # list while iterating
+            return self.jobs[n1-1:n2].copy()
+        else:
+            return [job_spec]
+
     def do_delete(self, job_id):
         """delete [job|*|n-m]
 Deletes the job specified. 'job' is either a job number or a job id.
 Use '*' to delete all the jobs. A range of jobs, specified by their numbers,
 can be deleted using 'n-m'.
 Uses current job if not specified."""
-        if job_id == '*':
-            # Update the list of jobs before trying to delete them
-            self._cache_jobs()
-            while self.jobs:
-                job = self.jobs[0]
-                self.delete_one_job(job)
-        elif m := re.fullmatch('(\\d+)-(\\d+)', job_id):
-            n1 = int(m.group(1))
-            n2 = int(m.group(2))
-            # We must gather the list of all the ids of jobs we want to delete
-            # right from the start, because deleting each job will update the
-            # list while iterating
-            ids = self.jobs[n1-1:n2]
-            for i in ids:
-                self.delete_one_job(i)
-        else:
-            self.delete_one_job(job_id)
+        jobs = self.parse_job_spec(job_id)
+        for j in jobs:
+            self.delete_one_job(j)
 
     def delete_one_job(self, job_id):
         previous_job = self.last_job_id
@@ -407,11 +417,15 @@ Uses current job if not specified."""
             self.last_job_id = self.jobs[0]
 
     def do_cancel(self, job_id):
-        """cancel [job]
-Stops the job specified.
-job is either a job number or a job id. Uses current job if not specified."""
-        job_id = self._get_and_remember_job_id(job_id)
-        self.lib.cancel_job(job_id)
+        """cancel [job|*|n-m]
+Stops the job specified. 'job' is either a job number or a job id.
+Use '*' to cancel all the jobs. A range of jobs, specified by their numbers,
+can be canceled using 'n-m'.
+Uses current job if not specified."""
+        jobs = self.parse_job_spec(job_id)
+        for j in jobs:
+            j = self._get_and_remember_job_id(j)
+            self.lib.cancel_job(j)
 
     @staticmethod
     def do_shell(command):
