@@ -163,6 +163,8 @@ class _CredentialsProvider:
             wml_credentials[self.REGION] = region
         if self.APIKEY not in wml_credentials and self.TOKEN not in wml_credentials:
             raise InvalidCredentials('API key (or token) must be specified.')
+        if self.APIKEY in wml_credentials and self.TOKEN in wml_credentials:
+            raise InvalidCredentials('API key and token must not be both specified.')
         if self.URL not in wml_credentials and self.REGION not in wml_credentials:
             raise InvalidCredentials('URL or region must be specified (but not both).')
         if self.APIKEY in wml_credentials:
@@ -335,6 +337,9 @@ class DOWMLLib:
         self._logger.warning('Attribute \'inline\' is deprecated: use \'inputs\' instead.')
         self.inputs = 'inline' if value else 'assets'
 
+    def is_connected_to_onprem(self):
+        return _CredentialsProvider.TOKEN in self._wml_credentials
+
     def _create_client(self):
         """Create the Python APIClient instance"""
         assert self._client is None
@@ -724,7 +729,10 @@ class DOWMLLib:
         wml_url = self._wml_credentials['url']
         # We don't want to (try to) delete the WS run if we only cancel the job
         # Instead, we skip everything until calling deployments.delete_job(True)
-        everything_ok_so_far = hard
+        # So 'hard' means that we try to delete the WS.
+        # Except if we are connected to CPD, as in this case we don't know how
+        # to find the WS URL.
+        everything_ok_so_far = hard and not self.is_connected_to_onprem()
         ws_url = client.PLATFORM_URLS_MAP.get(wml_url)
         if everything_ok_so_far and not ws_url:
             self._logger.error(f'Unknown Watson Studio URL for WML URL {wml_url}.')
@@ -1238,6 +1246,38 @@ class DOWMLLib:
         self._logger.debug(f'Model id: {model_id}')
         return model_id
 
+    def _create_space_metadata_for_cloud(self, client, wml_credentials):
+        cos_crn = _CredentialsProvider.COS_CRN
+        ml_crn = _CredentialsProvider.ML_CRN
+        if cos_crn not in wml_credentials or ml_crn not in wml_credentials:
+            raise NoCredentialsToCreateSpace(f'WML credentials do not contain the information necessary '
+                                             f'to create a deployment space. \nMissing \'{cos_crn}\' '
+                                             f'and/or \'{ml_crn}\'.')
+        assert type(wml_credentials[cos_crn]) is str
+        assert type(wml_credentials[ml_crn]) is str
+        csc = client.spaces.ConfigurationMetaNames
+        metadata = {
+            csc.NAME: self.space_name,
+            csc.DESCRIPTION: self.space_name + ' description',
+            csc.STORAGE: {
+                "type": "bmcos_object_storage",
+                "resource_crn": self._wml_credentials[cos_crn]
+            },
+            csc.COMPUTE: {
+                "name": "existing_instance_id",
+                "crn": self._wml_credentials[ml_crn]
+            }
+        }
+        return metadata
+
+    def _create_space_metadata_for_cpd(self, client):
+        csc = client.spaces.ConfigurationMetaNames
+        metadata = {
+            csc.NAME: self.space_name,
+            csc.DESCRIPTION: self.space_name + ' description',
+        }
+        return metadata
+
     def _find_or_create_space(self):
         """Find the Space to use from its name, create it if it doesn't exist."""
         assert self._client
@@ -1257,28 +1297,10 @@ class DOWMLLib:
             # Prepare necessary information
 
             wml_credentials = self._wml_credentials
-            cos_crn = _CredentialsProvider.COS_CRN
-            ml_crn = _CredentialsProvider.ML_CRN
-            if cos_crn not in wml_credentials or ml_crn not in wml_credentials:
-                raise NoCredentialsToCreateSpace(f'WML credentials do not contain the information necessary '
-                                                 f'to create a deployment space. \nMissing \'{cos_crn}\' '
-                                                 f'and/or \'{ml_crn}\'.')
-            assert type(wml_credentials[cos_crn]) is str
-            assert type(wml_credentials[ml_crn]) is str
-
-            csc = client.spaces.ConfigurationMetaNames
-            metadata = {
-                csc.NAME: self.space_name,
-                csc.DESCRIPTION: self.space_name + ' description',
-                csc.STORAGE: {
-                    "type": "bmcos_object_storage",
-                    "resource_crn": self._wml_credentials[cos_crn]
-                },
-                csc.COMPUTE: {
-                    "name": "existing_instance_id",
-                    "crn": self._wml_credentials[ml_crn]
-                }
-            }
+            if self.is_connected_to_onprem():
+                metadata = self._create_space_metadata_for_cpd(client)
+            else:
+                metadata = self._create_space_metadata_for_cloud(client, wml_credentials)
             # Create the space
             # We want the space to be ready as soon as the code returns from
             # spaces.store(), so we use background_mode=False. In addition, this
